// Global Header file of the CForth LowPower project ************************
//
// Author and (c)2020-24 Wolfgang Schemmert. The copyright claim is restricted
// to this file and the depending files "main.c", "main.h", "cforthLP.c" and
// "cforthLP.h"
// Other depending material is published, copyrighted and licensed
// by STMicroelectronics and Segger GmbH
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#ifndef GLOBALDEFINES
#define GLOBALDEFINES

#define REVNUM 12            //describes firmware revision

#define LQFP32				//else TSSOP20

#ifdef LQFP32
#define HARDWARE_VERSION 2  //describes project specific hardware version
#define LED_PB0				//=DEFAULT_LED in published PCB layout
//options, HW not published:
//#define MODUL_24	//24pin module w. quartz osc and RS-232 onboard.
					//LED pin is the only difference=PA11
//#define LED_PB8	//old 32pin version w.regulator,quartz,RS232driver on PCB
#else
#define HARDWARE_VERSION 1  //describes project specific hardware version
#define LED_PB8		//=DEFAULT_LED
//option, HW not published:
//#define MODUL_R	//20pin module w. quartz osc and RS-232 onboard.
					//LED pin is the only difference
#endif

//for LowPwr it makes sense only in very special cases:

#define MOTOR		//ifndef: PWM2 available

#define CLOCK_3M68 //default: HCLCK speed divided by 4 = 921.6kHz
					//SPEED=2:HCLK= 3.6864MHz
					//least supply current single speed mode
//#define CLOCK_14M	//default: HCLCK speed divided by 16 = 921.6kHz
					//SPEED=2:HCLK= 3.6864MHz SPEED=3:HCLK= 14.7456MHz
//#define CLOCK_48M	//special version with 8MHz crystal and 48MHz System Clock
					//else same hardware with RS-232 only. 
					//compared with Std version:No USB, more I/O and 2 presets
					//option CLOCK_2SPEED:divides 48MHz by 16=3MHz clock speed
					//default is 48MHz

#ifdef CLOCK_48M
#define PLLCLOCK
#endif

// *******************************

//attributes of KernelOperators
//attribute entries are merged with some other functional flags !!
#define A_IME			0x10000 //ATTRIBUTE_IMMEDIATE, cannot be compiled
#define A_CPO			0x20000 //ATTRIBUTE_COMPILEONLY,refuse work interpreting
#define A_BAK			0x40000 //may be parsed in background at executing state
#define A_CI			(A_CPO | A_IME) //COMPILEONLY IMMEDIATE
								//performs complex operations during compilation
#define A_IB			(A_IME | A_BAK)
#define A_RUN			0x80000 //ATTRIBUTE_RUNTIME,
					//compiled by other operators for special runtime behaviour
					//no direct user access, essential for viewing SEE and STEP
//only attribute of UserToken
#define A_VARCON		0x0C000000

#define ERR_STRINGTOKEN 1
#define ERR_SYNTAX		2
#define ERR_STACK		4
#define ERR_COMPONLY	8
#define ERR_SYSTEM		0x10
#define ERR_USERCODE	0x20
#define ERR_CONTEXT 	0x80
#define ERR_USERCONTEXT	0xA0	//combination of both, save code memory
#define ERR_USERSTRING	0x21	//combination of both, save code memory
#define ERR_CANCEL		0x100
#define ERR_RUNTOKEN	0x200
#define ERR_NUMVAL		0x400
#define ERR_DIVZERO		0x800

#define PAGE25 (0x08006400)	//globals: only gBaud, gfNumBase and gProject
#define PAGE26 (0x08006800)	//Project1:single word parameters and gfUserToken
#define PAGE27 (0x08006C00)	//Project1:gfUserToken and gfTCode
#define PAGE28 (0x08007000)	//Project1:gfTCode
#define PAGE29 (0x08007400)	//Project2:single word parameters and gfUserToken 
#define PAGE30 (0x08007800)	//Project2:gfUserToken and gfTCode
#define PAGE31 (0x08007C00)	//Project2:gfTCode

#define KERNELSIZE		126

#define MAXUSER			95//*4 words p.struct=380 words=1520 bytes=ca.1.5pages
#define MAXCODE			0x600	//bytes = 1.5 pages

#define MAXDATASTACK	28
#define MAXRETSTACK		28

#define CODETOP         MAXCODE - 1
#define USERTOP         (uint32_t)&gfUserHeader[MAXUSER]

#define USART_RXBUFLEN 512

#define DEFAULT_BAUD	11	//expanded to 115200, 
							//enter first 2 places, e.g. 96 for 9600
#define DEFAULT_NUMBASE	10 //Decimal, enter 16 for HEX

#define CTRL_B  2
#define CTRL_D  4
#define BKSP  8
#define TAB   9
#define LFF  10		//for Macintosh OS: simply exchange values of CRR and LFF
#define CRR  13		//then New_Line will be interpreted correctly
#define CTRL_R  18
#define ESC  27
#define SPACE  32
#define BACKSLASH 92

#endif  //GLOBALDEFINES

// ##########################################################################

// Version changes:
//REVNUM 1: 27 October2020 copied from CForth for STM32F042,REVNUM24.
//			no USB implemented here to save current and Flash space
//	08Nov20	return value of "interpretUserToken()" inverted
//			return of "interpretKernelToken()" changed to "void"
//			return of "executeTCode()" changed to "void"
//			opNODBG introduced,background process not executed,higher speed
//			if USART RX shorted during startup, AUTOEXE->"none"
//			(MINI: pull PA.. low during startup, AUTOEXE->"none")
//			subQuery() rejects (TIB bytes == 0), this simplifies scanNext()
//			return of scanNext() changed to "void", improvement of op_AUTO
//	19Nov20	special option CLOCK_48M, 
//			"SetSysClock()" moved from "system_stm32f0xx.c" to "main.c"
//	23Nov20	Kernel Ops DEBUG, CASE, <CASE new. ENDIF replaced by THEN
//release 03Dec20
//REVNUM 2:
//	05Dec20	executeTcode etc. optimized for runtime improvement
//			Z-BREAK renamed ZBREAK and doZ-BRK renamed doZBRK
//	06Dec20 IF()...LOOP() and doIF()...doLOOP() improved
//			for better runtime behaviour and better readability of MEM()
//			;(SEMIS) optimized
//	10Dec20	pullup/down switched off when I/O is output (save current)
//	11Dec20	op_LED modes reorganized (modes 0,1,2,3)
//			user terminal communication during NO-DEBUG improved (CTRL_D)
//	15Dec20	parameter format of sendX8() changed to uint32_t
//	16Dec20	time operators reorg. (MS, DS are blocking, BLOCK removed)		
//          improvement of CTRL_B (unblock MS and DS) behaviour
//	18Dec20 improvement of op_MS(),op_DECS(), CTRL_D works in MS and DECS
//release 28Dec20
//REVNUM 3:
//	14JAN21	Bugfix "RANA" PA2,3
//release 22Jan21
//REVNUM 4:
//	23Jan21	option CLOCK_3686K introduced (CPU clock = 3.6864 MHz)
//	13March21 gLedBusy init modified
//	MAY2021:
//	development system changed to "Segger Embedded Studio for ARM 5.44"
//	no changes of source code and project setup necessary
//	fundamental reorganisation for better use of limited memory size:
//	token code organisation changed to byte format 
//	takes more code, for this reason some less effective kernel ops removed
//	UserToken name length limited to 7char (+1 nullbyte)
//	optimization of op_REV (able to compile change of CONST or VAR)
//	AUTOEXE handling removed to "popFlashProject" (improves op_LOAD)
//	some minor improvement of program flow
//	19MAY21	Kernel ops ROLL and INJECT added, 2SWAP,2DUP,-ROT removed
//	22MAY21	%X and %D "Immediate,Background" now
//			op VAL added (reduced to Background Process value message)
//	24MAY21	MAXUSER now 95, MAXCODE reduced to 1536bytes for better balance
//	25May21	op_QUEST and AUTOEXE check moved to "popFlashProject" 
//			for better start of AUTOEXE when another project is loaded
//	30May21	PICK, ROLL, INJECT modfied to Forth79 style
//			FS ("flush data stack") modified to "NO attributes"
//	release 01June2021
//REVNUM 5:
//01Jun21:	bit7 of VAL,CONST token in "interpretUserToken()" not set any more
//			in doADDR, doVALUE and SEE not deleted any more
//20Jun21	new option LQFP32 uses PA10(RX) and PB6(TX) for USART1
//			and some more GPIO accessable
//			for more I/O ports some kernel Ops had to be removed (TIME,PULS)
//12Dec21	new option MOTOR instead of PWM2
//			bugfix popFlashProject, default gfSpeed adjustment
//13Dec21	speed option CLOCK_14M added: crystal 14.7456MHz
//17Dec21	new option CLOCK_2SPEED (for CLOCK_48M optionally)
//			kernel ops TIME and PULS removed
//			only 2 command sets: with CLOCK_2SPEED 
//			or without (instead includes REPLACE and RESTORE)
//			8MHz crystal replaced by 4MHz one (uses little bit less current)
//REVNUM 6:
//15APR22	new hardware option: LQFP32
//			48MHz versions: 4MHz crystal replaced by 8MHz (back to REVNUM < 5)
//release 18 April 2022
//REVNUM 7:
//08May22   %x renamed $, %d renamed %, REV renamed V
//			preloaded V2 removed, now P1,P2,P3 preloaded
//09MAY22	Bugfix OH,OL,IPU acceptance PA2,3,7
//			doBYLIT renamed doLIT8B
//16May22	op_PRINTSTACK modified (CRR removed)
//26May22	bugfix scanNext(): big number strings are interpreted falsely
//			compileUserSymbol() modified correspondingly
//			string output of op_USER modified
//28May22	gfTICKCOUNT -> gfMSCOUNT and gfTIXCOUNT (decouple TIX from  MS,DS)
//07June22	VAR and CONST replaced by VARCON,
//			op_REV compiles op_doREV instead of op_doADDR, saves runtime
//			op_VAL removed, op_TRACE new
//			P1,P2,P3 renamed to K1,K2,K3,K4
//			due to these changes, some reorganisation of code
//09June22	struct "gfUserToken" renamed to "gfUserHeader"
//11June22	bugfix interpretNumber(), parsing big negative numbers
//			kernel operator ARRAY added
//13June22	improvement of compileUserToken()
//			REPLACE and RESTORE removed due to lack of memory
//15June22	background process behaviour of KEY? and KEY improved
//16June22  bugfix eval  (uint32_t), gfTokennum is int32_t
//17June22	bugfix XNOP stack pointer syntax error
//release 17June2022
//REVNUM8:	18June2022
//22March23	kernel op "NEG" renamend to "+/-"
//			kernel op "=" renamend to "=="
//30Sept23	kernel op "%" renamend to "DZ"
//			kernel op "$" renamend to "HX"
//			kernel ops UNTIL and correseponding doUNTIL added
//			kernel ops PICK and TRACE deleted
//			option "Cold Reset" (CTRL_R) removed
//			independent Watchdog removed
//02Oct23	"RANA"(raed analog in) improved, new: disableADC(),saves current
//03Oct23	P renamed . , PH renamed .H , P" renamed ."
//			Kernel Op SLEEP implemented
//04Oct23	BSET, BCLR, BTST, .D implemented new
//			kernel Op 'Q' renamed to '?'
//07Oct23	numberBase saved in gerneral preset, always reactived at startup
//15Oct23	pin arrangement for PWM and MOT improved
//			kernel op '&' renamed to 'AND', kernel op '|' renamed to 'OR
//			kernel ops NOP and TRACE removed
//17Oct23	USER output reformatted
//18Oct23	bugfix and improvement/simplification of REPLACE, RESTORE
//21Oct23	MOT code improved
//25OCT23	RANA renamed RADC, RDIG renamed RDIN, WAIT renmed TIME
//26Oct23	kernel ops "<<" and ">>" replaced by single "SHIFT"
//27Oct23	single SPEED versions deleted, only 2 switchable clock speeds now
//			CONTI, BREAK, ZBREAK optimized, doCONTI,doBREAK,doZBREAK removed
//			K1,K2,K3,K4 renamed V1,V2,V3,V4
//30Oct23	servSystem() transferred from checkTerminal() to showPreStep()
//			gSYSCOUNT refresh rate 20Hz now, gLEDCOUNT adjusted to ca.300ms
//09Nov23	order of Kernel Ops rearranged
//12Nov23	RP-1 and RP-4 only "COMPILING" flag now.To be used ahead of RETURN
//release 12Nov23
//REVNUM 9:13Nov2023
//28Nov23	bugfix subPWM2()
//18Dec23	bugfix pushFlashProj(): gfBaseFreq instead of gfFreq
//19Dec23	op_MEM now sends block of 128bytes
//			MEM attributes changed to A_BAK
//			bugfix op_DOUNTIL. op_DOWHILE, op_DOLOOP
//20Dec23	bugfix compileTCode: protection against CODETOP overflow
//21Dec23	CheckValidIO improved
//23Dec23   op_QUEST(): printout of CPUclock improved
//released 25Dec23
//REVNUM->10
//30Dec23	bugfix compilation user header with kernel name (op_colon)
//31Dec23	new gfBackProcess=2: return to higher thread level with CTRL_R
//01Jan24	bugfix gfUserHeader[4] renamed to NSTP (confusion with op_MS)
//			op_MEM now sends block of 128bytes
//			improvement no countdown of NSTP if stepper is not rotating
//05Jan24	PWM range at 48M reduced to max 100, max.freq=30kHz
//released 10 Jan2024
//REVNUM-->11
//11Jan24	send0X32() modified: always with leading 0x even at small numbers
//18Jan24	bugfix op_SEE()
//19Jan24	decimal numbers can be entered with leading '%' at any muber base
//			see interpretNumber()
//06March24	ADC multiple read 4x average
//09March24	reorg kernel op SAVE and LOAD more Forth typically
//released 14 March2024
//REVNUM-->12
//31March24	bugfix cforth.h: DZ and HX have IMMEDIATE flag now
//01April24	kernelOps -ROT and ZBREAK replaced by REGW and REGR
//02Apr24	doVALUE renamed doVAL, doLIT8B renamed doLIT8
// *******************************